const axios = require('axios')
const agent = require('agentkeepalive')
const { ConcurrencyManager } = require('axios-concurrency')
stompjs = require('@stomp/stompjs');

if (typeof TextEncoder !== 'function') {
  const TextEncodingPolyfill = require('text-encoding');
  TextEncoder = TextEncodingPolyfill.TextEncoder;
  TextDecoder = TextEncodingPolyfill.TextDecoder;
}

Object.assign(global, { WebSocket: require('websocket').w3cwebsocket });


const TPAPI_MAX_CONCURRENT_REQUESTS = 5; 

class TPApi {
	constructor(base_url, username, password) 	{
		this.authToken = "";
		this.base_url = base_url;
		this.api_host = base_url.split('/')[2];
		this.username = username;
		this.password = password;
		this.stompClient = null;
		this.api_connection = axios.create({
			httpAgent: new agent({
			  maxSockets: 20,
			  maxFreeSockets: 5,
			  timeout: 60000, // active socket keepalive for 60 seconds
			  freeSocketTimeout: 30000, // free socket keepalive for 30 seconds	
			}),
			httpsAgent: new agent.HttpsAgent({
			  maxSockets: 20,
			  maxFreeSockets: 5,
			  timeout: 60000, // active socket keepalive for 60 seconds
			  freeSocketTimeout: 30000, // free socket keepalive for 30 seconds	
			})
		});
		this.manager = ConcurrencyManager(this.api_connection, TPAPI_MAX_CONCURRENT_REQUESTS);
	}
};

TPApi.prototype.api_connection = function() {};

TPApi.prototype.close = function() {
	this.manager.detach();
};

TPApi.prototype.getToken = function() {
	this_tpapi = this;

	return new Promise(function(resolve, reject){
		this_tpapi.api_connection({
			url: this_tpapi.base_url + '/api/v1/oauth/token',
			method: 'post',
			data: {
				userName: this_tpapi.username,
				password: this_tpapi.password
			}
		})
		.then(response => {
			//console.log(response);
			this_tpapi.authToken = response.data.token;
			resolve(response.data.token);
		})
		.catch(error => {
			//console.log(error);
			reject(error);
		});
	});
};

TPApi.prototype.getCompanybyCrmId = function(crmid) {
	this_tpapi = this;
	return new Promise(function(resolve, reject){
		this_tpapi.api_connection({
			url: this_tpapi.base_url + '/api/v1/compass/company/detail/' + crmid,
			method: 'get',
			headers: {
						Authorization: "Bearer " + this_tpapi.authToken
			}
		})
		.then(response => {
			//console.log(response.data);
			resolve(response.data);
		})
		.catch(error => {
			//console.log(error);
			reject(error);
		});
	});			

};

TPApi.prototype.getAssetsbyCompanyId = function(id) {
	this_tpapi = this;
	return new Promise(function(resolve, reject){
		this_tpapi.api_connection({
			url: this_tpapi.base_url + '/api/v1/compass/serviceAssets?companyId=' + id,
			method: 'get',
			headers: {
						Authorization: "Bearer " + this_tpapi.authToken
			}
		})
		.then(response => {
			//console.log(response.data);
			resolve(response.data);
		})
		.catch(error => {
			//console.log(error);
			reject(error);
		});
	});			

};

TPApi.prototype.getUserAttributes = function(id) {
	this_tpapi = this;
	return new Promise(function(resolve, reject){
		if(!id) {
			id = 'self';
		}
		this_tpapi.api_connection({
			url: this_tpapi.base_url + '/api/v1/authorization/user/' + id + '/attributes',
			method: 'get',
			headers: {
						Authorization: "Bearer " + this_tpapi.authToken
			}
		})
		.then(response => {
			//console.log(response.data);
			resolve(response.data);
		})
		.catch(error => {
			//console.log(error);
			reject(error);
		});
	});			

};

TPApi.prototype.getProfile = function(id) {
	this_tpapi = this;
	return new Promise(function(resolve, reject){
		if(!id) {
			id = 'self';
		}
		this_tpapi.api_connection({
			url: this_tpapi.base_url + '/api/v1/profile/myprofile',
			method: 'get',
			headers: {
						Authorization: "Bearer " + this_tpapi.authToken
			}
		})
		.then(response => {
			//console.log(response.data);
			resolve(response.data);
		})
		.catch(error => {
			//console.log(error);
			reject(error);
		});
	});			

};


TPApi.prototype.connectToStomp = function(topic, persist, messageCallback, errorCallback, debug) {
	this_tpapi = this;
	
	this_tpapi.stompClient = new stompjs.Client({
			brokerURL: 'wss://' + this_tpapi.api_host + '/notifications/websocket',
			connectHeaders: {
				login: this_tpapi.username,
				passcode: this_tpapi.authToken,
			},
			debug: function (str) {
				if (debug) {
					console.log(str);
				}
			},
			reconnectDelay: 5000,
			heartbeatIncoming: 4000,
			heartbeatOutgoing: 4000
	});

	this_tpapi.stompClient.onConnect = function(frame) {
		if (debug) {
			console.log('Connected to STOMP!');		
			console.log('Subscribing to Topic: ' + topic);
			console.log('Using Persistent Queue: ' + persist);
		}
		
		this_tpapi.getProfile()
		.then((response) => {
			var headers = null;
	
			if (persist) {
				headers = {
						id : response.ContactId,
						'x-queue-name' : response.ContactId,
						durable : true,
						'auto-delete' : false
				};
			} else {
				headers = {
					"id": "",
					"x-queue-name": "",
					"durable": false,
					"auto-delete": true
				};
			}
			this_tpapi.stompClient.subscribe(topic, messageCallback, headers);
		})
		.catch((error) => {
			console.log(error);
			
		})
		
	};

	this_tpapi.stompClient.onStompError = errorCallback;

	this_tpapi.stompClient.activate();

}

module.exports = TPApi;